﻿using Kivii.Seeyon.Entities;
using Kivii.Text;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Seeyon.Transforms
{
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "提交OA申请表单")]
    public class ApplyRequest : RestfulExecutionGeneric<Apply>
    {
        public string FormType { get; set; }

        public Guid Kvid { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            FormType.ThrowIfNull("请传入业务实体FormType");
            var type = Kivii.Text.AssemblyUtils.FindType(FormType);

            var instance = Activator.CreateInstance(type) as IEntityIsSeeyonApply;
            if (instance == null) return null;
            var results = instance.OnSubmitApplyForm<G>(Kvid);
            return results;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "提交OA申请表单")]
    public class ApplyRequest2 : RestfulExecutionGeneric<Apply>
    {
        public string FormType { get; set; }
        public JsonObject Form { get; set; }

        public override object OnExecutionGeneric<G>(IRequest req, IResponse res)
        {
            FormType.ThrowIfNull("请传入业务实体FormType");
            var type = Kivii.Text.AssemblyUtils.FindType(FormType);
            Dictionary<string,object> form = new Dictionary<string, object>();
            foreach (var item in Form)
            {
                if (item.Key == "Metadata")
                {
                    var metadata = JsonObject.Parse(item.Value);
                    JsonObject metadata2 = new JsonObject();
                    foreach (var meta in metadata)
                    {
                        var val = metadata[meta.Key];
                        metadata2[meta.Key] = val;
                    }
                    form[item.Key] = metadata2;
                    continue;
                }
                form[item.Key] = Form[item.Key];
            }
            var formFormat = form.ConvertTo<G>();
            var instance = Activator.CreateInstance(type) as IEntityIsSeeyonApply;
            if (instance == null) return null;
            var results = instance.OnSubmitApplyForm<G>(formFormat);
            return results;
        }
    }

    [Api(Description = "OA结果调用")]
    public class ApplyResult : RestfulExecution<Apply>
    {
        public string FormType { get; set; }

        public Guid Kvid { get; set; }

        public List<Guid> Kvids { get; set; }

        public object Result { get; set; }

        public string Message { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            FormType.ThrowIfNull("请传入业务实体FormType");
            var type = Kivii.Text.AssemblyUtils.FindType(FormType);

            var instance = Activator.CreateInstance(type) as IEntityIsSeeyonApply;
            if (instance == null) return null;
            if (Kvids.IsNullOrEmpty())
            {
                var results = instance.OnGetApplyResult(Kvid, Result, Message);
                return results;
            }
            else
            {
                var results = instance.OnGetApplyResult(Kvids, Result, Message);
                return results;
            }
        }
    }
}
